# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["ImageObject"]


class ImageObject(BaseModel):
    height: Optional[int] = None
    """The image height in pixels."""

    url: str
    """The source URL of the image."""

    width: Optional[int] = None
    """The image width in pixels."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from ..shared.image_object import ImageObject

__all__ = ["CategoryRetrieveResponse"]


class CategoryRetrieveResponse(BaseModel):
    id: str
    """
    The [Spotify category ID](/documentation/web-api/concepts/spotify-uris-ids) of
    the category.
    """

    href: str
    """A link to the Web API endpoint returning full details of the category."""

    icons: List[ImageObject]
    """The category icon, in various sizes."""

    name: str
    """The name of the category."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ...types.me import episode_list_params, episode_save_params, episode_check_params, episode_remove_params
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursorURLPage, AsyncCursorURLPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.me.episode_list_response import EpisodeListResponse
from ...types.me.episode_check_response import EpisodeCheckResponse

__all__ = ["EpisodesResource", "AsyncEpisodesResource"]


class EpisodesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EpisodesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return EpisodesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EpisodesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return EpisodesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        limit: int | Omit = omit,
        market: str | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorURLPage[EpisodeListResponse]:
        """
        Get a list of the episodes saved in the current Spotify user's library.<br/>
        This API endpoint is in **beta** and could change without warning. Please share
        any feedback that you have, or issues that you discover, in our
        [developer community forum](https://community.spotify.com/t5/Spotify-for-Developers/bd-p/Spotify_Developer).

        Args:
          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/me/episodes",
            page=SyncCursorURLPage[EpisodeListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "market": market,
                        "offset": offset,
                    },
                    episode_list_params.EpisodeListParams,
                ),
            ),
            model=EpisodeListResponse,
        )

    def check(
        self,
        *,
        ids: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EpisodeCheckResponse:
        """
        Check if one or more episodes is already saved in the current Spotify user's
        'Your Episodes' library.<br/> This API endpoint is in **beta** and could change
        without warning. Please share any feedback that you have, or issues that you
        discover, in our
        [developer community forum](https://community.spotify.com/t5/Spotify-for-Developers/bd-p/Spotify_Developer)..

        Args:
          ids: A comma-separated list of the
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids) for the
              episodes. Maximum: 50 IDs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/me/episodes/contains",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"ids": ids}, episode_check_params.EpisodeCheckParams),
            ),
            cast_to=EpisodeCheckResponse,
        )

    def remove(
        self,
        *,
        ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Remove one or more episodes from the current user's library.<br/> This API
        endpoint is in **beta** and could change without warning. Please share any
        feedback that you have, or issues that you discover, in our
        [developer community forum](https://community.spotify.com/t5/Spotify-for-Developers/bd-p/Spotify_Developer).

        Args:
          ids: A JSON array of the
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). <br/>A maximum
              of 50 items can be specified in one request. _**Note**: if the `ids` parameter
              is present in the query string, any IDs listed here in the body will be
              ignored._

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            "/me/episodes",
            body=maybe_transform({"ids": ids}, episode_remove_params.EpisodeRemoveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def save(
        self,
        *,
        ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Save one or more episodes to the current user's library.<br/> This API endpoint
        is in **beta** and could change without warning. Please share any feedback that
        you have, or issues that you discover, in our
        [developer community forum](https://community.spotify.com/t5/Spotify-for-Developers/bd-p/Spotify_Developer).

        Args:
          ids: A JSON array of the
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). <br/>A maximum
              of 50 items can be specified in one request. _**Note**: if the `ids` parameter
              is present in the query string, any IDs listed here in the body will be
              ignored._

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._put(
            "/me/episodes",
            body=maybe_transform({"ids": ids}, episode_save_params.EpisodeSaveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncEpisodesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEpisodesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncEpisodesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEpisodesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncEpisodesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        limit: int | Omit = omit,
        market: str | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[EpisodeListResponse, AsyncCursorURLPage[EpisodeListResponse]]:
        """
        Get a list of the episodes saved in the current Spotify user's library.<br/>
        This API endpoint is in **beta** and could change without warning. Please share
        any feedback that you have, or issues that you discover, in our
        [developer community forum](https://community.spotify.com/t5/Spotify-for-Developers/bd-p/Spotify_Developer).

        Args:
          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/me/episodes",
            page=AsyncCursorURLPage[EpisodeListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "market": market,
                        "offset": offset,
                    },
                    episode_list_params.EpisodeListParams,
                ),
            ),
            model=EpisodeListResponse,
        )

    async def check(
        self,
        *,
        ids: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EpisodeCheckResponse:
        """
        Check if one or more episodes is already saved in the current Spotify user's
        'Your Episodes' library.<br/> This API endpoint is in **beta** and could change
        without warning. Please share any feedback that you have, or issues that you
        discover, in our
        [developer community forum](https://community.spotify.com/t5/Spotify-for-Developers/bd-p/Spotify_Developer)..

        Args:
          ids: A comma-separated list of the
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids) for the
              episodes. Maximum: 50 IDs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/me/episodes/contains",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"ids": ids}, episode_check_params.EpisodeCheckParams),
            ),
            cast_to=EpisodeCheckResponse,
        )

    async def remove(
        self,
        *,
        ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Remove one or more episodes from the current user's library.<br/> This API
        endpoint is in **beta** and could change without warning. Please share any
        feedback that you have, or issues that you discover, in our
        [developer community forum](https://community.spotify.com/t5/Spotify-for-Developers/bd-p/Spotify_Developer).

        Args:
          ids: A JSON array of the
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). <br/>A maximum
              of 50 items can be specified in one request. _**Note**: if the `ids` parameter
              is present in the query string, any IDs listed here in the body will be
              ignored._

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            "/me/episodes",
            body=await async_maybe_transform({"ids": ids}, episode_remove_params.EpisodeRemoveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def save(
        self,
        *,
        ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Save one or more episodes to the current user's library.<br/> This API endpoint
        is in **beta** and could change without warning. Please share any feedback that
        you have, or issues that you discover, in our
        [developer community forum](https://community.spotify.com/t5/Spotify-for-Developers/bd-p/Spotify_Developer).

        Args:
          ids: A JSON array of the
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). <br/>A maximum
              of 50 items can be specified in one request. _**Note**: if the `ids` parameter
              is present in the query string, any IDs listed here in the body will be
              ignored._

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._put(
            "/me/episodes",
            body=await async_maybe_transform({"ids": ids}, episode_save_params.EpisodeSaveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class EpisodesResourceWithRawResponse:
    def __init__(self, episodes: EpisodesResource) -> None:
        self._episodes = episodes

        self.list = to_raw_response_wrapper(
            episodes.list,
        )
        self.check = to_raw_response_wrapper(
            episodes.check,
        )
        self.remove = to_raw_response_wrapper(
            episodes.remove,
        )
        self.save = to_raw_response_wrapper(
            episodes.save,
        )


class AsyncEpisodesResourceWithRawResponse:
    def __init__(self, episodes: AsyncEpisodesResource) -> None:
        self._episodes = episodes

        self.list = async_to_raw_response_wrapper(
            episodes.list,
        )
        self.check = async_to_raw_response_wrapper(
            episodes.check,
        )
        self.remove = async_to_raw_response_wrapper(
            episodes.remove,
        )
        self.save = async_to_raw_response_wrapper(
            episodes.save,
        )


class EpisodesResourceWithStreamingResponse:
    def __init__(self, episodes: EpisodesResource) -> None:
        self._episodes = episodes

        self.list = to_streamed_response_wrapper(
            episodes.list,
        )
        self.check = to_streamed_response_wrapper(
            episodes.check,
        )
        self.remove = to_streamed_response_wrapper(
            episodes.remove,
        )
        self.save = to_streamed_response_wrapper(
            episodes.save,
        )


class AsyncEpisodesResourceWithStreamingResponse:
    def __init__(self, episodes: AsyncEpisodesResource) -> None:
        self._episodes = episodes

        self.list = async_to_streamed_response_wrapper(
            episodes.list,
        )
        self.check = async_to_streamed_response_wrapper(
            episodes.check,
        )
        self.remove = async_to_streamed_response_wrapper(
            episodes.remove,
        )
        self.save = async_to_streamed_response_wrapper(
            episodes.save,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["PlaylistUpdateParams"]


class PlaylistUpdateParams(TypedDict, total=False):
    collaborative: bool
    """
    If `true`, the playlist will become collaborative and other users will be able
    to modify the playlist in their Spotify client. <br/> _**Note**: You can only
    set `collaborative` to `true` on non-public playlists._
    """

    description: str
    """
    Value for playlist description as displayed in Spotify Clients and in the Web
    API.
    """

    name: str
    """The new name for the playlist, for example `"My New Playlist Title"`"""

    public: bool
    """
    The playlist's public/private status (if it should be added to the user's
    profile or not): `true` the playlist will be public, `false` the playlist will
    be private, `null` the playlist status is not relevant. For more about
    public/private status, see
    [Working with Playlists](/documentation/web-api/concepts/playlists)
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PlayerSeekToPositionParams"]


class PlayerSeekToPositionParams(TypedDict, total=False):
    position_ms: Required[int]
    """The position in milliseconds to seek to.

    Must be a positive number. Passing in a position that is greater than the length
    of the track will cause the player to start playing the next song.
    """

    device_id: str
    """The id of the device this command is targeting.

    If not supplied, the user's currently active device is the target.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["ExternalURLObject"]


class ExternalURLObject(BaseModel):
    spotify: Optional[str] = None
    """
    The [Spotify URL](/documentation/web-api/concepts/spotify-uris-ids) for the
    object.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .shared.paging_playlist_object import PagingPlaylistObject

__all__ = ["BrowseGetFeaturedPlaylistsResponse"]


class BrowseGetFeaturedPlaylistsResponse(BaseModel):
    message: Optional[str] = None
    """The localized message of a playlist."""

    playlists: Optional[PagingPlaylistObject] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["ExternalIDObject"]


class ExternalIDObject(BaseModel):
    ean: Optional[str] = None
    """
    [International Article Number](http://en.wikipedia.org/wiki/International_Article_Number_%28EAN%29)
    """

    isrc: Optional[str] = None
    """
    [International Standard Recording Code](http://en.wikipedia.org/wiki/International_Standard_Recording_Code)
    """

    upc: Optional[str] = None
    """[Universal Product Code](http://en.wikipedia.org/wiki/Universal_Product_Code)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from spotted import Spotted, AsyncSpotted
from tests.utils import assert_matches_type
from spotted.types import (
    RecommendationGetResponse,
    RecommendationListAvailableGenreSeedsResponse,
)

# pyright: reportDeprecated=false

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestRecommendations:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            recommendation = client.recommendations.get()

        assert_matches_type(RecommendationGetResponse, recommendation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_with_all_params(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            recommendation = client.recommendations.get(
                limit=10,
                market="ES",
                max_acousticness=0,
                max_danceability=0,
                max_duration_ms=0,
                max_energy=0,
                max_instrumentalness=0,
                max_key=0,
                max_liveness=0,
                max_loudness=0,
                max_mode=0,
                max_popularity=0,
                max_speechiness=0,
                max_tempo=0,
                max_time_signature=0,
                max_valence=0,
                min_acousticness=0,
                min_danceability=0,
                min_duration_ms=0,
                min_energy=0,
                min_instrumentalness=0,
                min_key=0,
                min_liveness=0,
                min_loudness=0,
                min_mode=0,
                min_popularity=0,
                min_speechiness=0,
                min_tempo=0,
                min_time_signature=11,
                min_valence=0,
                seed_artists="4NHQUGzhtTLFvgF5SZesLK",
                seed_genres="classical,country",
                seed_tracks="0c6xIDDpzE81m2q797ordA",
                target_acousticness=0,
                target_danceability=0,
                target_duration_ms=0,
                target_energy=0,
                target_instrumentalness=0,
                target_key=0,
                target_liveness=0,
                target_loudness=0,
                target_mode=0,
                target_popularity=0,
                target_speechiness=0,
                target_tempo=0,
                target_time_signature=0,
                target_valence=0,
            )

        assert_matches_type(RecommendationGetResponse, recommendation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.recommendations.with_raw_response.get()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        recommendation = response.parse()
        assert_matches_type(RecommendationGetResponse, recommendation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with client.recommendations.with_streaming_response.get() as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                recommendation = response.parse()
                assert_matches_type(RecommendationGetResponse, recommendation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_available_genre_seeds(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            recommendation = client.recommendations.list_available_genre_seeds()

        assert_matches_type(RecommendationListAvailableGenreSeedsResponse, recommendation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_available_genre_seeds(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.recommendations.with_raw_response.list_available_genre_seeds()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        recommendation = response.parse()
        assert_matches_type(RecommendationListAvailableGenreSeedsResponse, recommendation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_available_genre_seeds(self, client: Spotted) -> None:
        with pytest.warns(DeprecationWarning):
            with client.recommendations.with_streaming_response.list_available_genre_seeds() as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                recommendation = response.parse()
                assert_matches_type(RecommendationListAvailableGenreSeedsResponse, recommendation, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncRecommendations:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            recommendation = await async_client.recommendations.get()

        assert_matches_type(RecommendationGetResponse, recommendation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_with_all_params(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            recommendation = await async_client.recommendations.get(
                limit=10,
                market="ES",
                max_acousticness=0,
                max_danceability=0,
                max_duration_ms=0,
                max_energy=0,
                max_instrumentalness=0,
                max_key=0,
                max_liveness=0,
                max_loudness=0,
                max_mode=0,
                max_popularity=0,
                max_speechiness=0,
                max_tempo=0,
                max_time_signature=0,
                max_valence=0,
                min_acousticness=0,
                min_danceability=0,
                min_duration_ms=0,
                min_energy=0,
                min_instrumentalness=0,
                min_key=0,
                min_liveness=0,
                min_loudness=0,
                min_mode=0,
                min_popularity=0,
                min_speechiness=0,
                min_tempo=0,
                min_time_signature=11,
                min_valence=0,
                seed_artists="4NHQUGzhtTLFvgF5SZesLK",
                seed_genres="classical,country",
                seed_tracks="0c6xIDDpzE81m2q797ordA",
                target_acousticness=0,
                target_danceability=0,
                target_duration_ms=0,
                target_energy=0,
                target_instrumentalness=0,
                target_key=0,
                target_liveness=0,
                target_loudness=0,
                target_mode=0,
                target_popularity=0,
                target_speechiness=0,
                target_tempo=0,
                target_time_signature=0,
                target_valence=0,
            )

        assert_matches_type(RecommendationGetResponse, recommendation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.recommendations.with_raw_response.get()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        recommendation = await response.parse()
        assert_matches_type(RecommendationGetResponse, recommendation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.recommendations.with_streaming_response.get() as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                recommendation = await response.parse()
                assert_matches_type(RecommendationGetResponse, recommendation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_available_genre_seeds(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            recommendation = await async_client.recommendations.list_available_genre_seeds()

        assert_matches_type(RecommendationListAvailableGenreSeedsResponse, recommendation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_available_genre_seeds(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.recommendations.with_raw_response.list_available_genre_seeds()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        recommendation = await response.parse()
        assert_matches_type(RecommendationListAvailableGenreSeedsResponse, recommendation, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_available_genre_seeds(self, async_client: AsyncSpotted) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.recommendations.with_streaming_response.list_available_genre_seeds() as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                recommendation = await response.parse()
                assert_matches_type(RecommendationListAvailableGenreSeedsResponse, recommendation, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions

import httpx

from ..types import recommendation_get_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.recommendation_get_response import RecommendationGetResponse
from ..types.recommendation_list_available_genre_seeds_response import RecommendationListAvailableGenreSeedsResponse

__all__ = ["RecommendationsResource", "AsyncRecommendationsResource"]


class RecommendationsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RecommendationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return RecommendationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RecommendationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return RecommendationsResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    def get(
        self,
        *,
        limit: int | Omit = omit,
        market: str | Omit = omit,
        max_acousticness: float | Omit = omit,
        max_danceability: float | Omit = omit,
        max_duration_ms: int | Omit = omit,
        max_energy: float | Omit = omit,
        max_instrumentalness: float | Omit = omit,
        max_key: int | Omit = omit,
        max_liveness: float | Omit = omit,
        max_loudness: float | Omit = omit,
        max_mode: int | Omit = omit,
        max_popularity: int | Omit = omit,
        max_speechiness: float | Omit = omit,
        max_tempo: float | Omit = omit,
        max_time_signature: int | Omit = omit,
        max_valence: float | Omit = omit,
        min_acousticness: float | Omit = omit,
        min_danceability: float | Omit = omit,
        min_duration_ms: int | Omit = omit,
        min_energy: float | Omit = omit,
        min_instrumentalness: float | Omit = omit,
        min_key: int | Omit = omit,
        min_liveness: float | Omit = omit,
        min_loudness: float | Omit = omit,
        min_mode: int | Omit = omit,
        min_popularity: int | Omit = omit,
        min_speechiness: float | Omit = omit,
        min_tempo: float | Omit = omit,
        min_time_signature: int | Omit = omit,
        min_valence: float | Omit = omit,
        seed_artists: str | Omit = omit,
        seed_genres: str | Omit = omit,
        seed_tracks: str | Omit = omit,
        target_acousticness: float | Omit = omit,
        target_danceability: float | Omit = omit,
        target_duration_ms: int | Omit = omit,
        target_energy: float | Omit = omit,
        target_instrumentalness: float | Omit = omit,
        target_key: int | Omit = omit,
        target_liveness: float | Omit = omit,
        target_loudness: float | Omit = omit,
        target_mode: int | Omit = omit,
        target_popularity: int | Omit = omit,
        target_speechiness: float | Omit = omit,
        target_tempo: float | Omit = omit,
        target_time_signature: int | Omit = omit,
        target_valence: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecommendationGetResponse:
        """
        Recommendations are generated based on the available information for a given
        seed entity and matched against similar artists and tracks. If there is
        sufficient information about the provided seeds, a list of tracks will be
        returned together with pool size details.

        For artists and tracks that are very new or obscure there might not be enough
        data to generate a list of tracks.

        Args:
          limit: The target size of the list of recommended tracks. For seeds with unusually
              small pools or when highly restrictive filtering is applied, it may be
              impossible to generate the requested number of recommended tracks. Debugging
              information for such cases is available in the response. Default: 20\\.. Minimum:
              1\\.. Maximum: 100.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          max_acousticness: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_danceability: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_duration_ms: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_energy: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_instrumentalness: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_key: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_liveness: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_loudness: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_mode: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_popularity: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_speechiness: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_tempo: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_time_signature: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_valence: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          min_acousticness: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_danceability: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_duration_ms: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_energy: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_instrumentalness: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_key: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_liveness: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_loudness: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_mode: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_popularity: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_speechiness: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_tempo: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_time_signature: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_valence: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          seed_artists: A comma separated list of
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids) for seed
              artists. Up to 5 seed values may be provided in any combination of
              `seed_artists`, `seed_tracks` and `seed_genres`.<br/> _**Note**: only required
              if `seed_genres` and `seed_tracks` are not set_.

          seed_genres: A comma separated list of any genres in the set of
              [available genre seeds](/documentation/web-api/reference/get-recommendation-genres).
              Up to 5 seed values may be provided in any combination of `seed_artists`,
              `seed_tracks` and `seed_genres`.<br/> _**Note**: only required if `seed_artists`
              and `seed_tracks` are not set_.

          seed_tracks: A comma separated list of
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids) for a seed
              track. Up to 5 seed values may be provided in any combination of `seed_artists`,
              `seed_tracks` and `seed_genres`.<br/> _**Note**: only required if `seed_artists`
              and `seed_genres` are not set_.

          target_acousticness: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_danceability: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_duration_ms: Target duration of the track (ms)

          target_energy: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_instrumentalness: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_key: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_liveness: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_loudness: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_mode: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_popularity: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_speechiness: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_tempo: Target tempo (BPM)

          target_time_signature: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_valence: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/recommendations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "market": market,
                        "max_acousticness": max_acousticness,
                        "max_danceability": max_danceability,
                        "max_duration_ms": max_duration_ms,
                        "max_energy": max_energy,
                        "max_instrumentalness": max_instrumentalness,
                        "max_key": max_key,
                        "max_liveness": max_liveness,
                        "max_loudness": max_loudness,
                        "max_mode": max_mode,
                        "max_popularity": max_popularity,
                        "max_speechiness": max_speechiness,
                        "max_tempo": max_tempo,
                        "max_time_signature": max_time_signature,
                        "max_valence": max_valence,
                        "min_acousticness": min_acousticness,
                        "min_danceability": min_danceability,
                        "min_duration_ms": min_duration_ms,
                        "min_energy": min_energy,
                        "min_instrumentalness": min_instrumentalness,
                        "min_key": min_key,
                        "min_liveness": min_liveness,
                        "min_loudness": min_loudness,
                        "min_mode": min_mode,
                        "min_popularity": min_popularity,
                        "min_speechiness": min_speechiness,
                        "min_tempo": min_tempo,
                        "min_time_signature": min_time_signature,
                        "min_valence": min_valence,
                        "seed_artists": seed_artists,
                        "seed_genres": seed_genres,
                        "seed_tracks": seed_tracks,
                        "target_acousticness": target_acousticness,
                        "target_danceability": target_danceability,
                        "target_duration_ms": target_duration_ms,
                        "target_energy": target_energy,
                        "target_instrumentalness": target_instrumentalness,
                        "target_key": target_key,
                        "target_liveness": target_liveness,
                        "target_loudness": target_loudness,
                        "target_mode": target_mode,
                        "target_popularity": target_popularity,
                        "target_speechiness": target_speechiness,
                        "target_tempo": target_tempo,
                        "target_time_signature": target_time_signature,
                        "target_valence": target_valence,
                    },
                    recommendation_get_params.RecommendationGetParams,
                ),
            ),
            cast_to=RecommendationGetResponse,
        )

    @typing_extensions.deprecated("deprecated")
    def list_available_genre_seeds(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecommendationListAvailableGenreSeedsResponse:
        """
        Retrieve a list of available genres seed parameter values for
        [recommendations](/documentation/web-api/reference/get-recommendations).
        """
        return self._get(
            "/recommendations/available-genre-seeds",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecommendationListAvailableGenreSeedsResponse,
        )


class AsyncRecommendationsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRecommendationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncRecommendationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRecommendationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncRecommendationsResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    async def get(
        self,
        *,
        limit: int | Omit = omit,
        market: str | Omit = omit,
        max_acousticness: float | Omit = omit,
        max_danceability: float | Omit = omit,
        max_duration_ms: int | Omit = omit,
        max_energy: float | Omit = omit,
        max_instrumentalness: float | Omit = omit,
        max_key: int | Omit = omit,
        max_liveness: float | Omit = omit,
        max_loudness: float | Omit = omit,
        max_mode: int | Omit = omit,
        max_popularity: int | Omit = omit,
        max_speechiness: float | Omit = omit,
        max_tempo: float | Omit = omit,
        max_time_signature: int | Omit = omit,
        max_valence: float | Omit = omit,
        min_acousticness: float | Omit = omit,
        min_danceability: float | Omit = omit,
        min_duration_ms: int | Omit = omit,
        min_energy: float | Omit = omit,
        min_instrumentalness: float | Omit = omit,
        min_key: int | Omit = omit,
        min_liveness: float | Omit = omit,
        min_loudness: float | Omit = omit,
        min_mode: int | Omit = omit,
        min_popularity: int | Omit = omit,
        min_speechiness: float | Omit = omit,
        min_tempo: float | Omit = omit,
        min_time_signature: int | Omit = omit,
        min_valence: float | Omit = omit,
        seed_artists: str | Omit = omit,
        seed_genres: str | Omit = omit,
        seed_tracks: str | Omit = omit,
        target_acousticness: float | Omit = omit,
        target_danceability: float | Omit = omit,
        target_duration_ms: int | Omit = omit,
        target_energy: float | Omit = omit,
        target_instrumentalness: float | Omit = omit,
        target_key: int | Omit = omit,
        target_liveness: float | Omit = omit,
        target_loudness: float | Omit = omit,
        target_mode: int | Omit = omit,
        target_popularity: int | Omit = omit,
        target_speechiness: float | Omit = omit,
        target_tempo: float | Omit = omit,
        target_time_signature: int | Omit = omit,
        target_valence: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecommendationGetResponse:
        """
        Recommendations are generated based on the available information for a given
        seed entity and matched against similar artists and tracks. If there is
        sufficient information about the provided seeds, a list of tracks will be
        returned together with pool size details.

        For artists and tracks that are very new or obscure there might not be enough
        data to generate a list of tracks.

        Args:
          limit: The target size of the list of recommended tracks. For seeds with unusually
              small pools or when highly restrictive filtering is applied, it may be
              impossible to generate the requested number of recommended tracks. Debugging
              information for such cases is available in the response. Default: 20\\.. Minimum:
              1\\.. Maximum: 100.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          max_acousticness: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_danceability: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_duration_ms: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_energy: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_instrumentalness: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_key: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_liveness: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_loudness: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_mode: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_popularity: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_speechiness: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_tempo: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_time_signature: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          max_valence: For each tunable track attribute, a hard ceiling on the selected track
              attribute’s value can be provided. See tunable track attributes below for the
              list of available options. For example, `max_instrumentalness=0.35` would filter
              out most tracks that are likely to be instrumental.

          min_acousticness: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_danceability: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_duration_ms: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_energy: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_instrumentalness: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_key: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_liveness: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_loudness: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_mode: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_popularity: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_speechiness: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_tempo: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_time_signature: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          min_valence: For each tunable track attribute, a hard floor on the selected track attribute’s
              value can be provided. See tunable track attributes below for the list of
              available options. For example, `min_tempo=140` would restrict results to only
              those tracks with a tempo of greater than 140 beats per minute.

          seed_artists: A comma separated list of
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids) for seed
              artists. Up to 5 seed values may be provided in any combination of
              `seed_artists`, `seed_tracks` and `seed_genres`.<br/> _**Note**: only required
              if `seed_genres` and `seed_tracks` are not set_.

          seed_genres: A comma separated list of any genres in the set of
              [available genre seeds](/documentation/web-api/reference/get-recommendation-genres).
              Up to 5 seed values may be provided in any combination of `seed_artists`,
              `seed_tracks` and `seed_genres`.<br/> _**Note**: only required if `seed_artists`
              and `seed_tracks` are not set_.

          seed_tracks: A comma separated list of
              [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids) for a seed
              track. Up to 5 seed values may be provided in any combination of `seed_artists`,
              `seed_tracks` and `seed_genres`.<br/> _**Note**: only required if `seed_artists`
              and `seed_genres` are not set_.

          target_acousticness: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_danceability: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_duration_ms: Target duration of the track (ms)

          target_energy: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_instrumentalness: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_key: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_liveness: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_loudness: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_mode: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_popularity: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_speechiness: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_tempo: Target tempo (BPM)

          target_time_signature: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          target_valence: For each of the tunable track attributes (below) a target value may be provided.
              Tracks with the attribute values nearest to the target values will be preferred.
              For example, you might request `target_energy=0.6` and
              `target_danceability=0.8`. All target values will be weighed equally in ranking
              results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/recommendations",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "market": market,
                        "max_acousticness": max_acousticness,
                        "max_danceability": max_danceability,
                        "max_duration_ms": max_duration_ms,
                        "max_energy": max_energy,
                        "max_instrumentalness": max_instrumentalness,
                        "max_key": max_key,
                        "max_liveness": max_liveness,
                        "max_loudness": max_loudness,
                        "max_mode": max_mode,
                        "max_popularity": max_popularity,
                        "max_speechiness": max_speechiness,
                        "max_tempo": max_tempo,
                        "max_time_signature": max_time_signature,
                        "max_valence": max_valence,
                        "min_acousticness": min_acousticness,
                        "min_danceability": min_danceability,
                        "min_duration_ms": min_duration_ms,
                        "min_energy": min_energy,
                        "min_instrumentalness": min_instrumentalness,
                        "min_key": min_key,
                        "min_liveness": min_liveness,
                        "min_loudness": min_loudness,
                        "min_mode": min_mode,
                        "min_popularity": min_popularity,
                        "min_speechiness": min_speechiness,
                        "min_tempo": min_tempo,
                        "min_time_signature": min_time_signature,
                        "min_valence": min_valence,
                        "seed_artists": seed_artists,
                        "seed_genres": seed_genres,
                        "seed_tracks": seed_tracks,
                        "target_acousticness": target_acousticness,
                        "target_danceability": target_danceability,
                        "target_duration_ms": target_duration_ms,
                        "target_energy": target_energy,
                        "target_instrumentalness": target_instrumentalness,
                        "target_key": target_key,
                        "target_liveness": target_liveness,
                        "target_loudness": target_loudness,
                        "target_mode": target_mode,
                        "target_popularity": target_popularity,
                        "target_speechiness": target_speechiness,
                        "target_tempo": target_tempo,
                        "target_time_signature": target_time_signature,
                        "target_valence": target_valence,
                    },
                    recommendation_get_params.RecommendationGetParams,
                ),
            ),
            cast_to=RecommendationGetResponse,
        )

    @typing_extensions.deprecated("deprecated")
    async def list_available_genre_seeds(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RecommendationListAvailableGenreSeedsResponse:
        """
        Retrieve a list of available genres seed parameter values for
        [recommendations](/documentation/web-api/reference/get-recommendations).
        """
        return await self._get(
            "/recommendations/available-genre-seeds",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RecommendationListAvailableGenreSeedsResponse,
        )


class RecommendationsResourceWithRawResponse:
    def __init__(self, recommendations: RecommendationsResource) -> None:
        self._recommendations = recommendations

        self.get = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                recommendations.get,  # pyright: ignore[reportDeprecated],
            )
        )
        self.list_available_genre_seeds = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                recommendations.list_available_genre_seeds,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncRecommendationsResourceWithRawResponse:
    def __init__(self, recommendations: AsyncRecommendationsResource) -> None:
        self._recommendations = recommendations

        self.get = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                recommendations.get,  # pyright: ignore[reportDeprecated],
            )
        )
        self.list_available_genre_seeds = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                recommendations.list_available_genre_seeds,  # pyright: ignore[reportDeprecated],
            )
        )


class RecommendationsResourceWithStreamingResponse:
    def __init__(self, recommendations: RecommendationsResource) -> None:
        self._recommendations = recommendations

        self.get = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                recommendations.get,  # pyright: ignore[reportDeprecated],
            )
        )
        self.list_available_genre_seeds = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                recommendations.list_available_genre_seeds,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncRecommendationsResourceWithStreamingResponse:
    def __init__(self, recommendations: AsyncRecommendationsResource) -> None:
        self._recommendations = recommendations

        self.get = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                recommendations.get,  # pyright: ignore[reportDeprecated],
            )
        )
        self.list_available_genre_seeds = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                recommendations.list_available_genre_seeds,  # pyright: ignore[reportDeprecated],
            )
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel
from .track_object import TrackObject
from .episode_object import EpisodeObject
from .playlist_user_object import PlaylistUserObject

__all__ = ["PlaylistTrackObject", "Track"]

Track: TypeAlias = Annotated[Union[TrackObject, EpisodeObject], PropertyInfo(discriminator="type")]


class PlaylistTrackObject(BaseModel):
    added_at: Optional[datetime] = None
    """The date and time the track or episode was added.

    _**Note**: some very old playlists may return `null` in this field._
    """

    added_by: Optional[PlaylistUserObject] = None
    """The Spotify user who added the track or episode.

    _**Note**: some very old playlists may return `null` in this field._
    """

    is_local: Optional[bool] = None
    """
    Whether this track or episode is a
    [local file](/documentation/web-api/concepts/playlists/#local-files) or not.
    """

    track: Optional[Track] = None
    """Information about the track or episode."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform
from ..._compat import cached_property
from ...types.me import top_list_top_tracks_params, top_list_top_artists_params
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursorURLPage, AsyncCursorURLPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.shared.track_object import TrackObject
from ...types.shared.artist_object import ArtistObject

__all__ = ["TopResource", "AsyncTopResource"]


class TopResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TopResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return TopResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TopResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return TopResourceWithStreamingResponse(self)

    def list_top_artists(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        time_range: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorURLPage[ArtistObject]:
        """
        Get the current user's top artists based on calculated affinity.

        Args:
          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          time_range: Over what time frame the affinities are computed. Valid values: `long_term`
              (calculated from ~1 year of data and including all new data as it becomes
              available), `medium_term` (approximately last 6 months), `short_term`
              (approximately last 4 weeks). Default: `medium_term`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/me/top/artists",
            page=SyncCursorURLPage[ArtistObject],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                        "time_range": time_range,
                    },
                    top_list_top_artists_params.TopListTopArtistsParams,
                ),
            ),
            model=ArtistObject,
        )

    def list_top_tracks(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        time_range: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorURLPage[TrackObject]:
        """
        Get the current user's top tracks based on calculated affinity.

        Args:
          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          time_range: Over what time frame the affinities are computed. Valid values: `long_term`
              (calculated from ~1 year of data and including all new data as it becomes
              available), `medium_term` (approximately last 6 months), `short_term`
              (approximately last 4 weeks). Default: `medium_term`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/me/top/tracks",
            page=SyncCursorURLPage[TrackObject],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                        "time_range": time_range,
                    },
                    top_list_top_tracks_params.TopListTopTracksParams,
                ),
            ),
            model=TrackObject,
        )


class AsyncTopResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTopResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncTopResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTopResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncTopResourceWithStreamingResponse(self)

    def list_top_artists(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        time_range: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ArtistObject, AsyncCursorURLPage[ArtistObject]]:
        """
        Get the current user's top artists based on calculated affinity.

        Args:
          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          time_range: Over what time frame the affinities are computed. Valid values: `long_term`
              (calculated from ~1 year of data and including all new data as it becomes
              available), `medium_term` (approximately last 6 months), `short_term`
              (approximately last 4 weeks). Default: `medium_term`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/me/top/artists",
            page=AsyncCursorURLPage[ArtistObject],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                        "time_range": time_range,
                    },
                    top_list_top_artists_params.TopListTopArtistsParams,
                ),
            ),
            model=ArtistObject,
        )

    def list_top_tracks(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        time_range: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[TrackObject, AsyncCursorURLPage[TrackObject]]:
        """
        Get the current user's top tracks based on calculated affinity.

        Args:
          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          offset: The index of the first item to return. Default: 0 (the first item). Use with
              limit to get the next set of items.

          time_range: Over what time frame the affinities are computed. Valid values: `long_term`
              (calculated from ~1 year of data and including all new data as it becomes
              available), `medium_term` (approximately last 6 months), `short_term`
              (approximately last 4 weeks). Default: `medium_term`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/me/top/tracks",
            page=AsyncCursorURLPage[TrackObject],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                        "time_range": time_range,
                    },
                    top_list_top_tracks_params.TopListTopTracksParams,
                ),
            ),
            model=TrackObject,
        )


class TopResourceWithRawResponse:
    def __init__(self, top: TopResource) -> None:
        self._top = top

        self.list_top_artists = to_raw_response_wrapper(
            top.list_top_artists,
        )
        self.list_top_tracks = to_raw_response_wrapper(
            top.list_top_tracks,
        )


class AsyncTopResourceWithRawResponse:
    def __init__(self, top: AsyncTopResource) -> None:
        self._top = top

        self.list_top_artists = async_to_raw_response_wrapper(
            top.list_top_artists,
        )
        self.list_top_tracks = async_to_raw_response_wrapper(
            top.list_top_tracks,
        )


class TopResourceWithStreamingResponse:
    def __init__(self, top: TopResource) -> None:
        self._top = top

        self.list_top_artists = to_streamed_response_wrapper(
            top.list_top_artists,
        )
        self.list_top_tracks = to_streamed_response_wrapper(
            top.list_top_tracks,
        )


class AsyncTopResourceWithStreamingResponse:
    def __init__(self, top: AsyncTopResource) -> None:
        self._top = top

        self.list_top_artists = async_to_streamed_response_wrapper(
            top.list_top_artists,
        )
        self.list_top_tracks = async_to_streamed_response_wrapper(
            top.list_top_tracks,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from ..._types import SequenceNotStr

__all__ = ["FollowingUnfollowParams"]


class FollowingUnfollowParams(TypedDict, total=False):
    ids: SequenceNotStr[str]
    """
    A JSON array of the artist or user
    [Spotify IDs](/documentation/web-api/concepts/spotify-uris-ids). For example:
    `{ids:["74ASZWbe4lXaubB36ztrGX", "08td7MxkoHQkXnWAYD8d6Q"]}`. A maximum of 50
    IDs can be sent in one request. _**Note**: if the `ids` parameter is present in
    the query string, any IDs listed here in the body will be ignored._
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..types import search_query_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.search_query_response import SearchQueryResponse

__all__ = ["SearchResource", "AsyncSearchResource"]


class SearchResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SearchResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return SearchResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SearchResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return SearchResourceWithStreamingResponse(self)

    def query(
        self,
        *,
        q: str,
        type: List[Literal["album", "artist", "playlist", "track", "show", "episode", "audiobook"]],
        include_external: Literal["audio"] | Omit = omit,
        limit: int | Omit = omit,
        market: str | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SearchQueryResponse:
        """
        Get Spotify catalog information about albums, artists, playlists, tracks, shows,
        episodes or audiobooks that match a keyword string. Audiobooks are only
        available within the US, UK, Canada, Ireland, New Zealand and Australia markets.

        Args:
          q: Your search query.

              You can narrow down your search using field filters. The available filters are
              `album`, `artist`, `track`, `year`, `upc`, `tag:hipster`, `tag:new`, `isrc`, and
              `genre`. Each field filter only applies to certain result types.

              The `artist` and `year` filters can be used while searching albums, artists and
              tracks. You can filter on a single `year` or a range (e.g. 1955-1960).<br /> The
              `album` filter can be used while searching albums and tracks.<br /> The `genre`
              filter can be used while searching artists and tracks.<br /> The `isrc` and
              `track` filters can be used while searching tracks.<br /> The `upc`, `tag:new`
              and `tag:hipster` filters can only be used while searching albums. The `tag:new`
              filter will return albums released in the past two weeks and `tag:hipster` can
              be used to return only albums with the lowest 10% popularity.<br />

          type: A comma-separated list of item types to search across. Search results include
              hits from all the specified item types. For example: `q=abacab&type=album,track`
              returns both albums and tracks matching "abacab".

          include_external: If `include_external=audio` is specified it signals that the client can play
              externally hosted audio content, and marks the content as playable in the
              response. By default externally hosted audio content is marked as unplayable in
              the response.

          limit: The maximum number of results to return in each item type.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          offset: The index of the first result to return. Use with limit to get the next page of
              search results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/search",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "q": q,
                        "type": type,
                        "include_external": include_external,
                        "limit": limit,
                        "market": market,
                        "offset": offset,
                    },
                    search_query_params.SearchQueryParams,
                ),
            ),
            cast_to=SearchQueryResponse,
        )


class AsyncSearchResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSearchResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncSearchResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSearchResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncSearchResourceWithStreamingResponse(self)

    async def query(
        self,
        *,
        q: str,
        type: List[Literal["album", "artist", "playlist", "track", "show", "episode", "audiobook"]],
        include_external: Literal["audio"] | Omit = omit,
        limit: int | Omit = omit,
        market: str | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SearchQueryResponse:
        """
        Get Spotify catalog information about albums, artists, playlists, tracks, shows,
        episodes or audiobooks that match a keyword string. Audiobooks are only
        available within the US, UK, Canada, Ireland, New Zealand and Australia markets.

        Args:
          q: Your search query.

              You can narrow down your search using field filters. The available filters are
              `album`, `artist`, `track`, `year`, `upc`, `tag:hipster`, `tag:new`, `isrc`, and
              `genre`. Each field filter only applies to certain result types.

              The `artist` and `year` filters can be used while searching albums, artists and
              tracks. You can filter on a single `year` or a range (e.g. 1955-1960).<br /> The
              `album` filter can be used while searching albums and tracks.<br /> The `genre`
              filter can be used while searching artists and tracks.<br /> The `isrc` and
              `track` filters can be used while searching tracks.<br /> The `upc`, `tag:new`
              and `tag:hipster` filters can only be used while searching albums. The `tag:new`
              filter will return albums released in the past two weeks and `tag:hipster` can
              be used to return only albums with the lowest 10% popularity.<br />

          type: A comma-separated list of item types to search across. Search results include
              hits from all the specified item types. For example: `q=abacab&type=album,track`
              returns both albums and tracks matching "abacab".

          include_external: If `include_external=audio` is specified it signals that the client can play
              externally hosted audio content, and marks the content as playable in the
              response. By default externally hosted audio content is marked as unplayable in
              the response.

          limit: The maximum number of results to return in each item type.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          offset: The index of the first result to return. Use with limit to get the next page of
              search results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/search",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "q": q,
                        "type": type,
                        "include_external": include_external,
                        "limit": limit,
                        "market": market,
                        "offset": offset,
                    },
                    search_query_params.SearchQueryParams,
                ),
            ),
            cast_to=SearchQueryResponse,
        )


class SearchResourceWithRawResponse:
    def __init__(self, search: SearchResource) -> None:
        self._search = search

        self.query = to_raw_response_wrapper(
            search.query,
        )


class AsyncSearchResourceWithRawResponse:
    def __init__(self, search: AsyncSearchResource) -> None:
        self._search = search

        self.query = async_to_raw_response_wrapper(
            search.query,
        )


class SearchResourceWithStreamingResponse:
    def __init__(self, search: SearchResource) -> None:
        self._search = search

        self.query = to_streamed_response_wrapper(
            search.query,
        )


class AsyncSearchResourceWithStreamingResponse:
    def __init__(self, search: AsyncSearchResource) -> None:
        self._search = search

        self.query = async_to_streamed_response_wrapper(
            search.query,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict

import httpx

from .queue import (
    QueueResource,
    AsyncQueueResource,
    QueueResourceWithRawResponse,
    AsyncQueueResourceWithRawResponse,
    QueueResourceWithStreamingResponse,
    AsyncQueueResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ....types.me import (
    player_transfer_params,
    player_get_state_params,
    player_skip_next_params,
    player_set_volume_params,
    player_skip_previous_params,
    player_pause_playback_params,
    player_start_playback_params,
    player_toggle_shuffle_params,
    player_set_repeat_mode_params,
    player_seek_to_position_params,
    player_list_recently_played_params,
    player_get_currently_playing_params,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncCursorURLPage, AsyncCursorURLPage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.me.player_get_state_response import PlayerGetStateResponse
from ....types.me.player_get_devices_response import PlayerGetDevicesResponse
from ....types.me.player_list_recently_played_response import PlayerListRecentlyPlayedResponse
from ....types.me.player_get_currently_playing_response import PlayerGetCurrentlyPlayingResponse

__all__ = ["PlayerResource", "AsyncPlayerResource"]


class PlayerResource(SyncAPIResource):
    @cached_property
    def queue(self) -> QueueResource:
        return QueueResource(self._client)

    @cached_property
    def with_raw_response(self) -> PlayerResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return PlayerResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PlayerResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return PlayerResourceWithStreamingResponse(self)

    def get_currently_playing(
        self,
        *,
        additional_types: str | Omit = omit,
        market: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlayerGetCurrentlyPlayingResponse:
        """
        Get the object currently being played on the user's Spotify account.

        Args:
          additional_types: A comma-separated list of item types that your client supports besides the
              default `track` type. Valid types are: `track` and `episode`.<br/> _**Note**:
              This parameter was introduced to allow existing clients to maintain their
              current behaviour and might be deprecated in the future._<br/> In addition to
              providing this parameter, make sure that your client properly handles cases of
              new types in the future by checking against the `type` field of each object.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/me/player/currently-playing",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "additional_types": additional_types,
                        "market": market,
                    },
                    player_get_currently_playing_params.PlayerGetCurrentlyPlayingParams,
                ),
            ),
            cast_to=PlayerGetCurrentlyPlayingResponse,
        )

    def get_devices(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlayerGetDevicesResponse:
        """Get information about a user’s available Spotify Connect devices.

        Some device
        models are not supported and will not be listed in the API response.
        """
        return self._get(
            "/me/player/devices",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlayerGetDevicesResponse,
        )

    def get_state(
        self,
        *,
        additional_types: str | Omit = omit,
        market: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlayerGetStateResponse:
        """
        Get information about the user’s current playback state, including track or
        episode, progress, and active device.

        Args:
          additional_types: A comma-separated list of item types that your client supports besides the
              default `track` type. Valid types are: `track` and `episode`.<br/> _**Note**:
              This parameter was introduced to allow existing clients to maintain their
              current behaviour and might be deprecated in the future._<br/> In addition to
              providing this parameter, make sure that your client properly handles cases of
              new types in the future by checking against the `type` field of each object.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/me/player",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "additional_types": additional_types,
                        "market": market,
                    },
                    player_get_state_params.PlayerGetStateParams,
                ),
            ),
            cast_to=PlayerGetStateResponse,
        )

    def list_recently_played(
        self,
        *,
        after: int | Omit = omit,
        before: int | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorURLPage[PlayerListRecentlyPlayedResponse]:
        """Get tracks from the current user's recently played tracks.

        _**Note**: Currently
        doesn't support podcast episodes._

        Args:
          after: A Unix timestamp in milliseconds. Returns all items after (but not including)
              this cursor position. If `after` is specified, `before` must not be specified.

          before: A Unix timestamp in milliseconds. Returns all items before (but not including)
              this cursor position. If `before` is specified, `after` must not be specified.

          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/me/player/recently-played",
            page=SyncCursorURLPage[PlayerListRecentlyPlayedResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "limit": limit,
                    },
                    player_list_recently_played_params.PlayerListRecentlyPlayedParams,
                ),
            ),
            model=PlayerListRecentlyPlayedResponse,
        )

    def pause_playback(
        self,
        *,
        device_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Pause playback on the user's account.

        This API only works for users who have
        Spotify Premium. The order of execution is not guaranteed when you use this API
        with other Player API endpoints.

        Args:
          device_id: The id of the device this command is targeting. If not supplied, the user's
              currently active device is the target.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._put(
            "/me/player/pause",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"device_id": device_id}, player_pause_playback_params.PlayerPausePlaybackParams),
            ),
            cast_to=NoneType,
        )

    def seek_to_position(
        self,
        *,
        position_ms: int,
        device_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Seeks to the given position in the user’s currently playing track.

        This API only
        works for users who have Spotify Premium. The order of execution is not
        guaranteed when you use this API with other Player API endpoints.

        Args:
          position_ms: The position in milliseconds to seek to. Must be a positive number. Passing in a
              position that is greater than the length of the track will cause the player to
              start playing the next song.

          device_id: The id of the device this command is targeting. If not supplied, the user's
              currently active device is the target.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._put(
            "/me/player/seek",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "position_ms": position_ms,
                        "device_id": device_id,
                    },
                    player_seek_to_position_params.PlayerSeekToPositionParams,
                ),
            ),
            cast_to=NoneType,
        )

    def set_repeat_mode(
        self,
        *,
        state: str,
        device_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Set the repeat mode for the user's playback.

        This API only works for users who
        have Spotify Premium. The order of execution is not guaranteed when you use this
        API with other Player API endpoints.

        Args:
          state: **track**, **context** or **off**.<br/> **track** will repeat the current
              track.<br/> **context** will repeat the current context.<br/> **off** will turn
              repeat off.

          device_id: The id of the device this command is targeting. If not supplied, the user's
              currently active device is the target.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._put(
            "/me/player/repeat",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "state": state,
                        "device_id": device_id,
                    },
                    player_set_repeat_mode_params.PlayerSetRepeatModeParams,
                ),
            ),
            cast_to=NoneType,
        )

    def set_volume(
        self,
        *,
        volume_percent: int,
        device_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Set the volume for the user’s current playback device.

        This API only works for
        users who have Spotify Premium. The order of execution is not guaranteed when
        you use this API with other Player API endpoints.

        Args:
          volume_percent: The volume to set. Must be a value from 0 to 100 inclusive.

          device_id: The id of the device this command is targeting. If not supplied, the user's
              currently active device is the target.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._put(
            "/me/player/volume",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "volume_percent": volume_percent,
                        "device_id": device_id,
                    },
                    player_set_volume_params.PlayerSetVolumeParams,
                ),
            ),
            cast_to=NoneType,
        )

    def skip_next(
        self,
        *,
        device_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Skips to next track in the user’s queue.

        This API only works for users who have
        Spotify Premium. The order of execution is not guaranteed when you use this API
        with other Player API endpoints.

        Args:
          device_id: The id of the device this command is targeting. If not supplied, the user's
              currently active device is the target.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            "/me/player/next",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"device_id": device_id}, player_skip_next_params.PlayerSkipNextParams),
            ),
            cast_to=NoneType,
        )

    def skip_previous(
        self,
        *,
        device_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Skips to previous track in the user’s queue.

        This API only works for users who
        have Spotify Premium. The order of execution is not guaranteed when you use this
        API with other Player API endpoints.

        Args:
          device_id: The id of the device this command is targeting. If not supplied, the user's
              currently active device is the target.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            "/me/player/previous",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"device_id": device_id}, player_skip_previous_params.PlayerSkipPreviousParams),
            ),
            cast_to=NoneType,
        )

    def start_playback(
        self,
        *,
        device_id: str | Omit = omit,
        context_uri: str | Omit = omit,
        offset: Dict[str, object] | Omit = omit,
        position_ms: int | Omit = omit,
        uris: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Start a new context or resume current playback on the user's active device.

        This
        API only works for users who have Spotify Premium. The order of execution is not
        guaranteed when you use this API with other Player API endpoints.

        Args:
          device_id: The id of the device this command is targeting. If not supplied, the user's
              currently active device is the target.

          context_uri: Optional. Spotify URI of the context to play. Valid contexts are albums, artists
              & playlists. `{context_uri:"spotify:album:1Je1IMUlBXcx1Fz0WE7oPT"}`

          offset: Optional. Indicates from where in the context playback should start. Only
              available when context_uri corresponds to an album or playlist object "position"
              is zero based and can’t be negative. Example: `"offset": {"position": 5}` "uri"
              is a string representing the uri of the item to start at. Example:
              `"offset": {"uri": "spotify:track:1301WleyT98MSxVHPZCA6M"}`

          position_ms: Indicates from what position to start playback. Must be a positive number.
              Passing in a position that is greater than the length of the track will cause
              the player to start playing the next song.

          uris:
              Optional. A JSON array of the Spotify track URIs to play. For example:
              `{"uris": ["spotify:track:4iV5W9uYEdYUVa79Axb7Rh", "spotify:track:1301WleyT98MSxVHPZCA6M"]}`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._put(
            "/me/player/play",
            body=maybe_transform(
                {
                    "context_uri": context_uri,
                    "offset": offset,
                    "position_ms": position_ms,
                    "uris": uris,
                },
                player_start_playback_params.PlayerStartPlaybackParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"device_id": device_id}, player_start_playback_params.PlayerStartPlaybackParams),
            ),
            cast_to=NoneType,
        )

    def toggle_shuffle(
        self,
        *,
        state: bool,
        device_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Toggle shuffle on or off for user’s playback.

        This API only works for users who
        have Spotify Premium. The order of execution is not guaranteed when you use this
        API with other Player API endpoints.

        Args:
          state: **true** : Shuffle user's playback.<br/> **false** : Do not shuffle user's
              playback.

          device_id: The id of the device this command is targeting. If not supplied, the user's
              currently active device is the target.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._put(
            "/me/player/shuffle",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "state": state,
                        "device_id": device_id,
                    },
                    player_toggle_shuffle_params.PlayerToggleShuffleParams,
                ),
            ),
            cast_to=NoneType,
        )

    def transfer(
        self,
        *,
        device_ids: SequenceNotStr[str],
        play: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Transfer playback to a new device and optionally begin playback.

        This API only
        works for users who have Spotify Premium. The order of execution is not
        guaranteed when you use this API with other Player API endpoints.

        Args:
          device_ids: A JSON array containing the ID of the device on which playback should be
              started/transferred.<br/>For
              example:`{device_ids:["74ASZWbe4lXaubB36ztrGX"]}`<br/>_**Note**: Although an
              array is accepted, only a single device_id is currently supported. Supplying
              more than one will return `400 Bad Request`_

          play:
              **true**: ensure playback happens on new device.<br/>**false** or not provided:
              keep the current playback state.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._put(
            "/me/player",
            body=maybe_transform(
                {
                    "device_ids": device_ids,
                    "play": play,
                },
                player_transfer_params.PlayerTransferParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncPlayerResource(AsyncAPIResource):
    @cached_property
    def queue(self) -> AsyncQueueResource:
        return AsyncQueueResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncPlayerResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cjavdev/spotted-py#accessing-raw-response-data-eg-headers
        """
        return AsyncPlayerResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPlayerResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cjavdev/spotted-py#with_streaming_response
        """
        return AsyncPlayerResourceWithStreamingResponse(self)

    async def get_currently_playing(
        self,
        *,
        additional_types: str | Omit = omit,
        market: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlayerGetCurrentlyPlayingResponse:
        """
        Get the object currently being played on the user's Spotify account.

        Args:
          additional_types: A comma-separated list of item types that your client supports besides the
              default `track` type. Valid types are: `track` and `episode`.<br/> _**Note**:
              This parameter was introduced to allow existing clients to maintain their
              current behaviour and might be deprecated in the future._<br/> In addition to
              providing this parameter, make sure that your client properly handles cases of
              new types in the future by checking against the `type` field of each object.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/me/player/currently-playing",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "additional_types": additional_types,
                        "market": market,
                    },
                    player_get_currently_playing_params.PlayerGetCurrentlyPlayingParams,
                ),
            ),
            cast_to=PlayerGetCurrentlyPlayingResponse,
        )

    async def get_devices(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlayerGetDevicesResponse:
        """Get information about a user’s available Spotify Connect devices.

        Some device
        models are not supported and will not be listed in the API response.
        """
        return await self._get(
            "/me/player/devices",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PlayerGetDevicesResponse,
        )

    async def get_state(
        self,
        *,
        additional_types: str | Omit = omit,
        market: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PlayerGetStateResponse:
        """
        Get information about the user’s current playback state, including track or
        episode, progress, and active device.

        Args:
          additional_types: A comma-separated list of item types that your client supports besides the
              default `track` type. Valid types are: `track` and `episode`.<br/> _**Note**:
              This parameter was introduced to allow existing clients to maintain their
              current behaviour and might be deprecated in the future._<br/> In addition to
              providing this parameter, make sure that your client properly handles cases of
              new types in the future by checking against the `type` field of each object.

          market: An
              [ISO 3166-1 alpha-2 country code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2).
              If a country code is specified, only content that is available in that market
              will be returned.<br/> If a valid user access token is specified in the request
              header, the country associated with the user account will take priority over
              this parameter.<br/> _**Note**: If neither market or user country are provided,
              the content is considered unavailable for the client._<br/> Users can view the
              country that is associated with their account in the
              [account settings](https://www.spotify.com/account/overview/).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/me/player",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "additional_types": additional_types,
                        "market": market,
                    },
                    player_get_state_params.PlayerGetStateParams,
                ),
            ),
            cast_to=PlayerGetStateResponse,
        )

    def list_recently_played(
        self,
        *,
        after: int | Omit = omit,
        before: int | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PlayerListRecentlyPlayedResponse, AsyncCursorURLPage[PlayerListRecentlyPlayedResponse]]:
        """Get tracks from the current user's recently played tracks.

        _**Note**: Currently
        doesn't support podcast episodes._

        Args:
          after: A Unix timestamp in milliseconds. Returns all items after (but not including)
              this cursor position. If `after` is specified, `before` must not be specified.

          before: A Unix timestamp in milliseconds. Returns all items before (but not including)
              this cursor position. If `before` is specified, `after` must not be specified.

          limit: The maximum number of items to return. Default: 20. Minimum: 1. Maximum: 50.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/me/player/recently-played",
            page=AsyncCursorURLPage[PlayerListRecentlyPlayedResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "limit": limit,
                    },
                    player_list_recently_played_params.PlayerListRecentlyPlayedParams,
                ),
            ),
            model=PlayerListRecentlyPlayedResponse,
        )

    async def pause_playback(
        self,
        *,
        device_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Pause playback on the user's account.

        This API only works for users who have
        Spotify Premium. The order of execution is not guaranteed when you use this API
        with other Player API endpoints.

        Args:
          device_id: The id of the device this command is targeting. If not supplied, the user's
              currently active device is the target.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._put(
            "/me/player/pause",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"device_id": device_id}, player_pause_playback_params.PlayerPausePlaybackParams
                ),
            ),
            cast_to=NoneType,
        )

    async def seek_to_position(
        self,
        *,
        position_ms: int,
        device_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Seeks to the given position in the user’s currently playing track.

        This API only
        works for users who have Spotify Premium. The order of execution is not
        guaranteed when you use this API with other Player API endpoints.

        Args:
          position_ms: The position in milliseconds to seek to. Must be a positive number. Passing in a
              position that is greater than the length of the track will cause the player to
              start playing the next song.

          device_id: The id of the device this command is targeting. If not supplied, the user's
              currently active device is the target.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._put(
            "/me/player/seek",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "position_ms": position_ms,
                        "device_id": device_id,
                    },
                    player_seek_to_position_params.PlayerSeekToPositionParams,
                ),
            ),
            cast_to=NoneType,
        )

    async def set_repeat_mode(
        self,
        *,
        state: str,
        device_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Set the repeat mode for the user's playback.

        This API only works for users who
        have Spotify Premium. The order of execution is not guaranteed when you use this
        API with other Player API endpoints.

        Args:
          state: **track**, **context** or **off**.<br/> **track** will repeat the current
              track.<br/> **context** will repeat the current context.<br/> **off** will turn
              repeat off.

          device_id: The id of the device this command is targeting. If not supplied, the user's
              currently active device is the target.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._put(
            "/me/player/repeat",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "state": state,
                        "device_id": device_id,
                    },
                    player_set_repeat_mode_params.PlayerSetRepeatModeParams,
                ),
            ),
            cast_to=NoneType,
        )

    async def set_volume(
        self,
        *,
        volume_percent: int,
        device_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Set the volume for the user’s current playback device.

        This API only works for
        users who have Spotify Premium. The order of execution is not guaranteed when
        you use this API with other Player API endpoints.

        Args:
          volume_percent: The volume to set. Must be a value from 0 to 100 inclusive.

          device_id: The id of the device this command is targeting. If not supplied, the user's
              currently active device is the target.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._put(
            "/me/player/volume",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "volume_percent": volume_percent,
                        "device_id": device_id,
                    },
                    player_set_volume_params.PlayerSetVolumeParams,
                ),
            ),
            cast_to=NoneType,
        )

    async def skip_next(
        self,
        *,
        device_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Skips to next track in the user’s queue.

        This API only works for users who have
        Spotify Premium. The order of execution is not guaranteed when you use this API
        with other Player API endpoints.

        Args:
          device_id: The id of the device this command is targeting. If not supplied, the user's
              currently active device is the target.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            "/me/player/next",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"device_id": device_id}, player_skip_next_params.PlayerSkipNextParams
                ),
            ),
            cast_to=NoneType,
        )

    async def skip_previous(
        self,
        *,
        device_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Skips to previous track in the user’s queue.

        This API only works for users who
        have Spotify Premium. The order of execution is not guaranteed when you use this
        API with other Player API endpoints.

        Args:
          device_id: The id of the device this command is targeting. If not supplied, the user's
              currently active device is the target.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            "/me/player/previous",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"device_id": device_id}, player_skip_previous_params.PlayerSkipPreviousParams
                ),
            ),
            cast_to=NoneType,
        )

    async def start_playback(
        self,
        *,
        device_id: str | Omit = omit,
        context_uri: str | Omit = omit,
        offset: Dict[str, object] | Omit = omit,
        position_ms: int | Omit = omit,
        uris: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Start a new context or resume current playback on the user's active device.

        This
        API only works for users who have Spotify Premium. The order of execution is not
        guaranteed when you use this API with other Player API endpoints.

        Args:
          device_id: The id of the device this command is targeting. If not supplied, the user's
              currently active device is the target.

          context_uri: Optional. Spotify URI of the context to play. Valid contexts are albums, artists
              & playlists. `{context_uri:"spotify:album:1Je1IMUlBXcx1Fz0WE7oPT"}`

          offset: Optional. Indicates from where in the context playback should start. Only
              available when context_uri corresponds to an album or playlist object "position"
              is zero based and can’t be negative. Example: `"offset": {"position": 5}` "uri"
              is a string representing the uri of the item to start at. Example:
              `"offset": {"uri": "spotify:track:1301WleyT98MSxVHPZCA6M"}`

          position_ms: Indicates from what position to start playback. Must be a positive number.
              Passing in a position that is greater than the length of the track will cause
              the player to start playing the next song.

          uris:
              Optional. A JSON array of the Spotify track URIs to play. For example:
              `{"uris": ["spotify:track:4iV5W9uYEdYUVa79Axb7Rh", "spotify:track:1301WleyT98MSxVHPZCA6M"]}`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._put(
            "/me/player/play",
            body=await async_maybe_transform(
                {
                    "context_uri": context_uri,
                    "offset": offset,
                    "position_ms": position_ms,
                    "uris": uris,
                },
                player_start_playback_params.PlayerStartPlaybackParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"device_id": device_id}, player_start_playback_params.PlayerStartPlaybackParams
                ),
            ),
            cast_to=NoneType,
        )

    async def toggle_shuffle(
        self,
        *,
        state: bool,
        device_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Toggle shuffle on or off for user’s playback.

        This API only works for users who
        have Spotify Premium. The order of execution is not guaranteed when you use this
        API with other Player API endpoints.

        Args:
          state: **true** : Shuffle user's playback.<br/> **false** : Do not shuffle user's
              playback.

          device_id: The id of the device this command is targeting. If not supplied, the user's
              currently active device is the target.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._put(
            "/me/player/shuffle",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "state": state,
                        "device_id": device_id,
                    },
                    player_toggle_shuffle_params.PlayerToggleShuffleParams,
                ),
            ),
            cast_to=NoneType,
        )

    async def transfer(
        self,
        *,
        device_ids: SequenceNotStr[str],
        play: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Transfer playback to a new device and optionally begin playback.

        This API only
        works for users who have Spotify Premium. The order of execution is not
        guaranteed when you use this API with other Player API endpoints.

        Args:
          device_ids: A JSON array containing the ID of the device on which playback should be
              started/transferred.<br/>For
              example:`{device_ids:["74ASZWbe4lXaubB36ztrGX"]}`<br/>_**Note**: Although an
              array is accepted, only a single device_id is currently supported. Supplying
              more than one will return `400 Bad Request`_

          play:
              **true**: ensure playback happens on new device.<br/>**false** or not provided:
              keep the current playback state.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._put(
            "/me/player",
            body=await async_maybe_transform(
                {
                    "device_ids": device_ids,
                    "play": play,
                },
                player_transfer_params.PlayerTransferParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class PlayerResourceWithRawResponse:
    def __init__(self, player: PlayerResource) -> None:
        self._player = player

        self.get_currently_playing = to_raw_response_wrapper(
            player.get_currently_playing,
        )
        self.get_devices = to_raw_response_wrapper(
            player.get_devices,
        )
        self.get_state = to_raw_response_wrapper(
            player.get_state,
        )
        self.list_recently_played = to_raw_response_wrapper(
            player.list_recently_played,
        )
        self.pause_playback = to_raw_response_wrapper(
            player.pause_playback,
        )
        self.seek_to_position = to_raw_response_wrapper(
            player.seek_to_position,
        )
        self.set_repeat_mode = to_raw_response_wrapper(
            player.set_repeat_mode,
        )
        self.set_volume = to_raw_response_wrapper(
            player.set_volume,
        )
        self.skip_next = to_raw_response_wrapper(
            player.skip_next,
        )
        self.skip_previous = to_raw_response_wrapper(
            player.skip_previous,
        )
        self.start_playback = to_raw_response_wrapper(
            player.start_playback,
        )
        self.toggle_shuffle = to_raw_response_wrapper(
            player.toggle_shuffle,
        )
        self.transfer = to_raw_response_wrapper(
            player.transfer,
        )

    @cached_property
    def queue(self) -> QueueResourceWithRawResponse:
        return QueueResourceWithRawResponse(self._player.queue)


class AsyncPlayerResourceWithRawResponse:
    def __init__(self, player: AsyncPlayerResource) -> None:
        self._player = player

        self.get_currently_playing = async_to_raw_response_wrapper(
            player.get_currently_playing,
        )
        self.get_devices = async_to_raw_response_wrapper(
            player.get_devices,
        )
        self.get_state = async_to_raw_response_wrapper(
            player.get_state,
        )
        self.list_recently_played = async_to_raw_response_wrapper(
            player.list_recently_played,
        )
        self.pause_playback = async_to_raw_response_wrapper(
            player.pause_playback,
        )
        self.seek_to_position = async_to_raw_response_wrapper(
            player.seek_to_position,
        )
        self.set_repeat_mode = async_to_raw_response_wrapper(
            player.set_repeat_mode,
        )
        self.set_volume = async_to_raw_response_wrapper(
            player.set_volume,
        )
        self.skip_next = async_to_raw_response_wrapper(
            player.skip_next,
        )
        self.skip_previous = async_to_raw_response_wrapper(
            player.skip_previous,
        )
        self.start_playback = async_to_raw_response_wrapper(
            player.start_playback,
        )
        self.toggle_shuffle = async_to_raw_response_wrapper(
            player.toggle_shuffle,
        )
        self.transfer = async_to_raw_response_wrapper(
            player.transfer,
        )

    @cached_property
    def queue(self) -> AsyncQueueResourceWithRawResponse:
        return AsyncQueueResourceWithRawResponse(self._player.queue)


class PlayerResourceWithStreamingResponse:
    def __init__(self, player: PlayerResource) -> None:
        self._player = player

        self.get_currently_playing = to_streamed_response_wrapper(
            player.get_currently_playing,
        )
        self.get_devices = to_streamed_response_wrapper(
            player.get_devices,
        )
        self.get_state = to_streamed_response_wrapper(
            player.get_state,
        )
        self.list_recently_played = to_streamed_response_wrapper(
            player.list_recently_played,
        )
        self.pause_playback = to_streamed_response_wrapper(
            player.pause_playback,
        )
        self.seek_to_position = to_streamed_response_wrapper(
            player.seek_to_position,
        )
        self.set_repeat_mode = to_streamed_response_wrapper(
            player.set_repeat_mode,
        )
        self.set_volume = to_streamed_response_wrapper(
            player.set_volume,
        )
        self.skip_next = to_streamed_response_wrapper(
            player.skip_next,
        )
        self.skip_previous = to_streamed_response_wrapper(
            player.skip_previous,
        )
        self.start_playback = to_streamed_response_wrapper(
            player.start_playback,
        )
        self.toggle_shuffle = to_streamed_response_wrapper(
            player.toggle_shuffle,
        )
        self.transfer = to_streamed_response_wrapper(
            player.transfer,
        )

    @cached_property
    def queue(self) -> QueueResourceWithStreamingResponse:
        return QueueResourceWithStreamingResponse(self._player.queue)


class AsyncPlayerResourceWithStreamingResponse:
    def __init__(self, player: AsyncPlayerResource) -> None:
        self._player = player

        self.get_currently_playing = async_to_streamed_response_wrapper(
            player.get_currently_playing,
        )
        self.get_devices = async_to_streamed_response_wrapper(
            player.get_devices,
        )
        self.get_state = async_to_streamed_response_wrapper(
            player.get_state,
        )
        self.list_recently_played = async_to_streamed_response_wrapper(
            player.list_recently_played,
        )
        self.pause_playback = async_to_streamed_response_wrapper(
            player.pause_playback,
        )
        self.seek_to_position = async_to_streamed_response_wrapper(
            player.seek_to_position,
        )
        self.set_repeat_mode = async_to_streamed_response_wrapper(
            player.set_repeat_mode,
        )
        self.set_volume = async_to_streamed_response_wrapper(
            player.set_volume,
        )
        self.skip_next = async_to_streamed_response_wrapper(
            player.skip_next,
        )
        self.skip_previous = async_to_streamed_response_wrapper(
            player.skip_previous,
        )
        self.start_playback = async_to_streamed_response_wrapper(
            player.start_playback,
        )
        self.toggle_shuffle = async_to_streamed_response_wrapper(
            player.toggle_shuffle,
        )
        self.transfer = async_to_streamed_response_wrapper(
            player.transfer,
        )

    @cached_property
    def queue(self) -> AsyncQueueResourceWithStreamingResponse:
        return AsyncQueueResourceWithStreamingResponse(self._player.queue)
